# Copyright 2014-2016, A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

import unittest2 as unittest

from acos_client import errors as acos_errors
from acos_client.v30.slb import virtual_server


EXPECTED_URL = '/axapi/v3/slb/virtual-server/'


class TestVirtualServer(unittest.TestCase):
    def setUp(self):
        self.client = mock.MagicMock()
        self.target = virtual_server.VirtualServer(self.client)
        self.target._get = mock.MagicMock(side_effect=acos_errors.NotFound)

    def test_arp_disable_positive(self):
        self._test_arp_disable(True)

    def test_arp_disable_negative(self):
        self._test_arp_disable(False)

    def _build_payload(self, arp_disable=False):
        return {'virtual-server': {'arp-disable': None if arp_disable is None else int(arp_disable),
                                   'name': 'virtualserver', 'ip-address': '127.0.0.1'}}

    def _test_arp_disable(self, arp_disable):
        self.target.create("virtualserver", "127.0.0.1",
                           arp_disable=arp_disable)
        call_source = self.client.http
        expected_payload = self._build_payload(arp_disable=arp_disable)
        call_source.request.assert_called_with('POST', EXPECTED_URL, expected_payload, mock.ANY)

    def test_oper(self):
        client = mock.MagicMock()
        vs = virtual_server.VirtualServer(client)
        vsname = "fake-loadbalancer"
        vs.oper(vsname)

        ((method, url, params, header), kwargs) = client.http.request.call_args
        expected_url = "/axapi/v3/slb/virtual-server/{0}/oper".format(vsname)
        self.assertEqual(method, "GET")
        self.assertEqual(expected_url, url)

    def test_stats(self):
        client = mock.MagicMock()
        vs = virtual_server.VirtualServer(client)
        vsname = "fake-loadbalancer"
        vs.stats(vsname)
        ((method, url, params, header), kwargs) = client.http.request.call_args
        expected_url = "/axapi/v3/slb/virtual-server/{0}/port/stats".format(vsname)
        self.assertEqual(method, "GET")
        self.assertEqual(expected_url, url)



# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.errors as ae
import acos_client.v30.base as base


class VirtualPort(base.BaseV30):

    # Protocols
    TCP = "tcp"
    UDP = "udp"
    OTHERS = "others"
    DIAMETER = "diameter"
    DNS_TCP = "dns-tcp"
    DNS_UDP = "dns-udp"
    FAST_HTTP = "fast-http"
    FIX = "fix"
    FTP = "ftp"
    FTP_PROXY = "ftp-proxy"
    HTTP = "http"
    HTTPS = "https"
    MLB = "mlb"
    MMS = "mms"
    MYSQL = "mysql"
    MSSQL = "mssql"
    RADIUS = "radius"
    RTSP = "rtsp"
    SIP = "sip"
    SIP_TCP = "sip-tcp"
    SIPS = "sips"
    SMPP_TCP = "smpp-tcp"
    SPDY = "spdy"
    SPDYS = "spdys"
    SMTP = "smtp"
    SSL_PROXY = "ssl-proxy"
    TCP_PROXY = "tcp-proxy"
    TFTP = "tftp"
    GENERIC_PROXY = "tcp-proxy"

    url_server_tmpl = '/slb/virtual-server/{name}/port/'
    url_port_tmpl = '{port_number}+{protocol}'

    def all(self, virtual_server_name, **kwargs):
        url = self.url_server_tmpl.format(name=virtual_server_name)
        return self._get(url, **kwargs)

    def get(self, virtual_server_name, name, protocol, port):
        url = self.url_server_tmpl.format(name=virtual_server_name)
        url += self.url_port_tmpl.format(
            port_number=port, protocol=protocol
        )
        return self._get(url)

    def _set(self, virtual_server_name, name, protocol, port,
             service_group_name,
             s_pers_name=None, c_pers_name=None, stats=0, update=False,
             no_dest_nat=None,
             exclude_minimize=[],
             autosnat=False,
             ipinip=False,
             pool=None,
             **kwargs):

        params = {
            "port": self.minimal_dict({
                "name": name,
                "service-group": service_group_name,
                "protocol": protocol,
                "port-number": int(port),
                "template-persist-source-ip": s_pers_name,
                "template-persist-cookie": c_pers_name,
                "extended-stats": stats
            }, exclude=exclude_minimize)
        }
        if autosnat:
            params['port']['auto'] = int(autosnat)
        if ipinip:
            params['port']['ipinip'] = int(ipinip)
        if pool and len(pool) > 0:
            params['port']['pool'] = pool

        server_ssl_tmpl = kwargs.get("template_server_ssl", None)
        client_ssl_tmpl = kwargs.get("template_client_ssl")

        if server_ssl_tmpl:
            params['port']['template-server-ssl'] = server_ssl_tmpl
        if client_ssl_tmpl:
            params['port']['template-client-ssl'] = client_ssl_tmpl

        sampling_enable = kwargs.get('sampling_enable')
        if sampling_enable is not None:
            self._set_sampling_enable(sampling_enable, params)

        if no_dest_nat is not None:
            params["port"]["no-dest-nat"] = 1 if no_dest_nat else 0

        url = self.url_server_tmpl.format(name=virtual_server_name)
        if update:
            url += self.url_port_tmpl.format(
                port_number=port, protocol=protocol
            )

        return self._post(url, params, **kwargs)

    def create(self, virtual_server_name, name, protocol, port,
               service_group_name,
               s_pers_name=None, c_pers_name=None, status=1,
               autosnat=False,
               ipinip=False,
               no_dest_nat=None,
               source_nat_pool=None,
               **kwargs):
        return self._set(virtual_server_name,
                         name, protocol, port, service_group_name,
                         s_pers_name, c_pers_name, status,
                         autosnat=autosnat, ipinip=ipinip,
                         no_dest_nat=no_dest_nat, pool=source_nat_pool,
                         **kwargs)

    def update(self, virtual_server_name, name, protocol, port,
               service_group_name,
               s_pers_name=None, c_pers_name=None, status=1,
               autosnat=False,
               ipinip=False,
               no_dest_nat=None,
               source_nat_pool=None,
               **kwargs):
        vp = self.get(virtual_server_name, name, protocol, port)
        if vp is None:
            raise ae.NotFound()

        exclu = ['template-persist-source-ip', 'template-persist-cookie']

        try:
            return self._set(virtual_server_name,
                             name, protocol, port, service_group_name,
                             s_pers_name, c_pers_name, status, True,
                             autosnat=autosnat, ipinip=ipinip,
                             exclude_minimize=exclu, no_dest_nat=no_dest_nat,
                             pool=source_nat_pool,
                             **kwargs)
        except ae.AxapiJsonFormatError:
            return self._set(virtual_server_name,
                             name, protocol, port, service_group_name,
                             s_pers_name, c_pers_name, status, True,
                             autosnat=autosnat, ipinip=ipinip,
                             exclude_minimize=[], no_dest_nat=no_dest_nat,
                             pool=source_nat_pool,
                             **kwargs)

    def delete(self, virtual_server_name, name, protocol, port):
        url = self.url_server_tmpl.format(name=virtual_server_name)
        url += self.url_port_tmpl.format(port_number=port, protocol=protocol)
        return self._delete(url)

    def _set_sampling_enable(self, sample_list, dest_obj):
        dest_array = []
        for x in sample_list:
            entry = {"counters1": x}
            dest_array.append(entry)

        dest_obj["port"]["sampling-enable"] = dest_array



# Copyright 2015, A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.errors as acos_errors
import acos_client.v30.base as base


class Nat(base.BaseV30):
    @property
    def pool(self):
        return self.Pool(self.client)

    class Pool(base.BaseV30):
        url_prefix = "/ip/nat/pool/"

        def _set(self, name, start_ip, end_ip, mask, **kwargs):
            params = {
                "pool": self.minimal_dict({
                    'pool-name': name,
                    'start-address': start_ip,
                    'end-address': end_ip,
                    'netmask': mask,
                    }),
            }
            self._post(self.url_prefix + name, params, **kwargs)

        def get(self, name):
            return self._get(self.url_prefix + name)

        def all(self):
            return self._get(self.url_prefix)

        def create(self, name, start_ip, end_ip, mask, **kwargs):
            try:
                self.get(name)
            except acos_errors.NotFound:
                pass
            else:
                raise acos_errors.Exists
            self._set(name, start_ip, end_ip, mask, **kwargs)

        def delete(self, name, **kwargs):
            self._delete(self.url_prefix + name)

        def stats(self, name='', **kwargs):
            return self._get(self.url_prefix + name + '/stats', **kwargs)

        def all_stats(self, **kwargs):
            return self.stats()



# Copyright 2015,  Tobit Raff,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.v30.base as base

from ssl_cert import SSLCert
from ssl_key import SSLKey


class File(base.BaseV30):
    @property
    def ssl_cert(self):
        return SSLCert(self.client)

    @property
    def ssl_key(self):
        return SSLKey(self.client)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.v21.base as base


class VirtualPort(base.BaseV21):

    # Protocols
    TCP = 2
    UDP = 3
    HTTP = 11
    HTTPS = 12
    OTHERS = 4
    RTSP = 5
    FTP = 6
    MMS = 7
    SIP = 8
    FAST_HTTP = 9
    GENERIC_PROXY = 10
    SSL_PROXY = 13
    SMTP = 14
    SIP_TCP = 15
    SIPS = 16
    DIAMETER = 17
    DNS_UDP = 18
    TFTP = 19
    DNS_TCP = 20
    RADIUS = 21
    MYSQL = 22
    MSSQL = 23
    FIX = 24
    SMPP_TCP = 25
    SPDY = 26
    SPDYS = 27
    FTP_PROXY = 28

    # The keys as specified in the ACOS JSON message.
    CLIENT_SSL_TMPL_KEY = "client_ssl_template"
    SERVER_SSL_TMPL_KEY = "server_ssl_template"

    # The keys as sent from a10-neutron-lbaas
    # They match what we use in v4 so we transform here
    CLIENT_SSL_ANL_KEY = "template_client_ssl"
    SERVER_SSL_ANL_KEY = "template_server_ssl"

    def _set(self, action, virtual_server_name, name, protocol, port,
             service_group_name,
             s_pers_name=None, c_pers_name=None, status=1,
             autosnat=False,
             ipinip=False,
             source_nat=None,
             **kwargs):
        params = {
            "name": virtual_server_name,
            "vport": self.minimal_dict({
                "name": name,
                "service_group": service_group_name,
                "protocol": protocol,
                "port": int(port),
                "source_ip_persistence_template": s_pers_name,
                "cookie_persistence_template": c_pers_name,
                "status": status
            })
        }

        client_ssl_template = kwargs.get(self.CLIENT_SSL_TMPL_KEY)
        server_ssl_template = kwargs.get(self.SERVER_SSL_TMPL_KEY)

        if client_ssl_template:
            params['vport'][self.CLIENT_SSL_ANL_KEY] = client_ssl_template

        if server_ssl_template:
            params['vport'][self.SERVER_SSL_ANL_KEY] = server_ssl_template

        if autosnat:
            params['vport']['source_nat_auto'] = int(autosnat)
        if ipinip:
            params['vport']['ip_in_ip'] = int(ipinip)
        if source_nat and len(source_nat) > 0:
            params['vport']['source_nat'] = source_nat

        self._post(action, params, **kwargs)

    def get(self, virtual_server_name, name, protocol, port, **kwargs):
        # There is no slb.virtual_server.vport.search.
        # Instead, we get the virtual server and get the desired vport.
        results = self._post('slb.virtual_server.search', {'name': virtual_server_name}, **kwargs)

        vports = results.get("virtual_server").get("vport_list", [])
        filtered_vports = filter(lambda x: x.get("name") == name, vports)
        if len(filtered_vports) > 0:
            return filtered_vports[0]

    def create(self, virtual_server_name, name, protocol, port,
               service_group_name,
               s_pers_name=None, c_pers_name=None, status=1,
               autosnat=False,
               ipinip=False,
               source_nat_pool=None,
               **kwargs):
        self._set('slb.virtual_server.vport.create', virtual_server_name,
                  name, protocol, port, service_group_name,
                  s_pers_name, c_pers_name, status,
                  autosnat=autosnat, ipinip=ipinip, source_nat=source_nat_pool,
                  **kwargs)

    def update(self, virtual_server_name, name, protocol, port,
               service_group_name,
               s_pers_name=None, c_pers_name=None, status=1,
               autosnat=False,
               ipinip=False,
               source_nat_pool=None,
               **kwargs):
            self._set('slb.virtual_server.vport.update', virtual_server_name,
                      name, protocol, port, service_group_name,
                      s_pers_name, c_pers_name, status,
                      autosnat=autosnat, ipinip=ipinip, source_nat=source_nat_pool,
                      **kwargs)

    def delete(self, virtual_server_name, name, protocol, port, **kwargs):
        params = {
            "name": virtual_server_name,
            "vport": {
                "name": name,
                "protocol": protocol,
                "port": int(port)
            }
        }
        self._post("slb.virtual_server.vport.delete", params, **kwargs)



# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import acos_client.v30.base as base

from persistence import CookiePersistence
from persistence import SourceIpPersistence
from ssl import ClientSSL
from ssl import ServerSSL


class Template(base.BaseV30):

    @property
    def client_ssl(self):
        return ClientSSL(self.client)

    @property
    def cookie_persistence(self):
        return CookiePersistence(self.client)

    @property
    def src_ip_persistence(self):
        return SourceIpPersistence(self.client)

    @property
    def server_ssl(self):
        return ServerSSL(self.client)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
import unittest2 as unittest

from acos_client import errors as acos_errors
from acos_client.v30.slb import member


class TestMember(unittest.TestCase):
    def setUp(self):
        self.client = mock.MagicMock()
        self.member = member.Member(self.client)
        self.member._get = mock.MagicMock(side_effect=acos_errors.NotFound)

        # common test parameter(s) throughout all test-cases
        self._sg_name = 'fake-sg'

    def test_create_enable_member(self):
        expected = {
            'member': {
                'name': 'fake-srever',
                'port': 80,
                'member-stats-data-disable': self.member.STATUS_ENABLE,
                'member-state': 'enable',
            }
        }
        self.member.create(self._sg_name, 'fake-srever', 80)

        ((method, url, params, header), kwargs) = self.client.http.request.call_args

        self.assertEqual(url, '/axapi/v3/slb/service-group/%s/member/' % (self._sg_name))
        self.assertEqual(params, expected)

    def test_create_disable_member(self):
        expected = {
            'member': {
                'name': 'fake-srever',
                'port': 80,
                'member-stats-data-disable': self.member.STATUS_DISABLE,
                'member-state': 'disable',
            }
        }
        self.member.create(self._sg_name, 'fake-srever', 80, self.member.STATUS_DISABLE, False)

        ((method, url, params, header), kwargs) = self.client.http.request.call_args

        self.assertEqual(params, expected)

    def test_update_enable_member(self):
        expected = {
            'member': {
                'name': 'fake-srever',
                'port': 443,
                'member-stats-data-disable': self.member.STATUS_ENABLE,
                'member-state': 'enable',
            }
        }
        self.member.update(self._sg_name, 'fake-srever', 443)

        ((method, url, params, header), kwargs) = self.client.http.request.call_args

        self.assertEqual(url, '/axapi/v3/slb/service-group/%s/member/%s+%s/' % 
                              (self._sg_name,
                               expected['member']['name'],
                               expected['member']['port']))
        self.assertEqual(params, expected)

    def test_update_disable_member(self):
        expected = {
            'member': {
                'name': 'fake-srever',
                'port': 443,
                'member-stats-data-disable': self.member.STATUS_DISABLE,
                'member-state': 'disable',
            }
        }
        self.member.update(self._sg_name, 'fake-srever', 443, self.member.STATUS_DISABLE, False)

        ((method, url, params, header), kwargs) = self.client.http.request.call_args

        self.assertEqual(url, '/axapi/v3/slb/service-group/%s/member/%s+%s/' % 
                              (self._sg_name,
                               expected['member']['name'],
                               expected['member']['port']))
        self.assertEqual(params, expected)
